from .ask_dubo import *  # noqa
from .config import DOCS_URL, FEEDBACK_URL


__title__ = "dubo"
__description__ = "Natural language analytics on any data frame or database."
__version__ = "0.2.8"


MSG = f"Dubo | Examples: {DOCS_URL} | Contact: {FEEDBACK_URL} | Privacy policy: https://mercator.tech/privacy"

print(MSG)
